function kSmootherOut = kfMultipleLags(x,SystemMatrices,nL)
% calculates autocovariance of states at different lags 
% P_{t,t-l|T}=E[(S_{t}-S_{t|T})(S_{t-l}-S_{t-l|T})']

%standardize data
xNaN =bsxfun(@minus,  x,   SystemMatrices.mX); 
xNaN =bsxfun(@rdivide,xNaN,SystemMatrices.vX);

%run smoother
kFilterOut   =kalman_filter(xNaN,SystemMatrices);
kSmootherOut =kalman_smoother(kFilterOut);

%unpack estimates
SystemMatrices =kFilterOut.SystemMatrices; 
A =SystemMatrices.A; 
Q =SystemMatrices.Q; 
C =SystemMatrices.C;


P_filter =kFilterOut.P_filter;    P_filter{1} =[];  P_filter =P_filter(~cellfun(@isempty,P_filter));
P_smooth =kSmootherOut.P_smooth;  P_smooth{1} =[];  P_smooth =P_smooth(~cellfun(@isempty,P_smooth));

S_smooth =kSmootherOut.S_smooth(:,2:end)';


T =length(P_smooth); 
nS=length(A); 

%calculate autocovariances
P_smoothNl   =cell(nL+1,1); 
P_smoothNl{1}=reshape(cell2mat(P_smooth'),nS,nS,T); 

%stationary VAR: \Gamma(h)=A*\Gamma(h-1)
for t=T:-1:nL+1
    for l=1:nL
        
        P_smoothNl{l+1}(:,:,t)=(P_filter{t-l}*A'*pinv(A*P_filter{t-l}*A'+Q))*P_smoothNl{l}(:,:,t);
    end
end

%load output structure
kSmootherOut.P_smoothNl =P_smoothNl;
kSmootherOut.x_smooth   =S_smooth*C';
kSmootherOut.X_smooth   =repmat(SystemMatrices.vX,T,1).*kSmootherOut.x_smooth+...
    repmat(SystemMatrices.mX,T,1);